/*****
 ForgetMeNot, Space Heater
*****/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#include "DHT.h"

// DHT 22 sensor
#define DHTPIN D1
#define DHTTYPE DHT22

// DHT sensor
DHT dht(DHTPIN, DHTTYPE);

// Change the credentials below, so your ESP8266 connects to your network. USE A HOTSPOT! Proper WiFi does not work for some reason
const char* ssid = "<WIFI_SSID>";
const char* password = "<WIFI_PASSWORD>";

// Run Mosquitto, do "ifconfig" and get the IP address of your laptop on the HOTSPOT's interface, and add it here
const char* mqtt_server = "<YOUR_LOCAL_MQTT_BROKER_IP>";
 
// Initializes the espClient
WiFiClient espClient;
PubSubClient client(espClient);

// 0 Means the heater is off
int currentHeatStatus = 0;

// Connect your ESP8266 to WiFi
void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  Serial.begin(9600);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

// This functions reconnects your ESP8266 to your MQTT broker
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Heat")) {
      Serial.println("connected");
      // Get activation messages
      client.subscribe("house/heat");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 1 seconds");
      // Wait 5 seconds before retrying
      delay(1000);
    }
  }
}

// The setup function sets your ESP GPIOs to Outputs, starts the serial communication at a baud rate of 9600, and sets your MQTT broker
void setup() {
  Serial.begin(9600);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  dht.begin();
}

void loop() {
  // This part insures your ESP is connected to your broker
  if (!client.connected()) {
    reconnect();
  }

  // Measure the temperature when your space heater is on and define the threshold below accordingly
  // In our case, above 30 celsius degrees the space heater is on
  int heatVal = (int) dht.readTemperature();
  int heatStatus = heatVal > 30 ? 1 : 0;
  char* msg;

  if(heatStatus != currentHeatStatus){
    currentHeatStatus = heatStatus;
    if(currentHeatStatus == 1){
      msg = "on";
    }else{
      msg = "off";
    }
    // client.publish Sends the message to the relevant component, in our case - house/heat
    client.publish("house/heat",msg);
    Serial.print("sent heat is  ");
    Serial.println(msg);
    delay(500);
    
  }
  Serial.println(heatVal);
} 